package org.example.mislugares;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.StrictMode;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnInfoWindowClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

/**
 * @author josue.ferri
 * 
 */
public class MainActivity extends FragmentActivity implements LocationListener,
		OnInfoWindowClickListener {

	public AdaptadorLugares adaptador;
	private LocationManager manejador;
	private Location mejorLocaliz;
	private GoogleMap mapa;
	static Friend misDatos;
	static SharedPreferences pref;
	static boolean usuario = false;
	SharedPreferences.Editor editor;
	static Almacen almacen;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		StrictMode.setThreadPolicy(new StrictMode.ThreadPolicy.Builder()
				.permitNetwork().build());
		pref = PreferenceManager.getDefaultSharedPreferences(this);

		if (pref.getString("name", "?") == "?") {
			lanzarMisDatos(null);
		} else {
			misDatos = new Friend(pref.getString("name", "?"), pref.getString(
					"mail", "?"), pref.getFloat("longitud", 0), pref.getFloat(
					"latitud", 0));
			Log.e("obtener datos de pref",
					pref.getString("name", "?") + pref.getString("mail", "?")
							+ pref.getFloat("longitud", 0)
							+ pref.getFloat("latitud", 0));
			usuario = true;
		}

		almacen = new AlmacenPHP2();
		manejador = (LocationManager) getSystemService(LOCATION_SERVICE);

		if (usuario) {
			if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
				actualizaMejorLocaliz(manejador
						.getLastKnownLocation(LocationManager.GPS_PROVIDER));
			}
			if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
				actualizaMejorLocaliz(manejador
						.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
			}
		}

		setContentView(R.layout.mapa);
		mapa = ((SupportMapFragment) getSupportFragmentManager()
				.findFragmentById(R.id.mapa)).getMap();
		mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);
		mapa.setMyLocationEnabled(true);
		mapa.getUiSettings().setZoomControlsEnabled(true);
		mapa.getUiSettings().setCompassEnabled(true);

		Friends.vectorFriends = almacen.getFriends();

		for (Friend lugar : Friends.vectorFriends) {
			GeoPunto p = lugar.getPosicion();
			if (p != null && p.getLatitud() != 0) {
				BitmapDrawable iconoDrawable = (BitmapDrawable) getResources()
						.getDrawable(lugar.getFoto());
				Bitmap iGrande = iconoDrawable.getBitmap();
				Bitmap icono = Bitmap.createScaledBitmap(iGrande,
						iGrande.getWidth() / 7, iGrande.getHeight() / 7, false);
				mapa.addMarker(new MarkerOptions()
						.position(new LatLng(p.getLatitud(), p.getLongitud()))
						.title(lugar.getNombre()).snippet(lugar.getDireccion())
						.icon(BitmapDescriptorFactory.fromBitmap(icono)));
			}
		}
		mapa.setOnInfoWindowClickListener(this);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.about:
			lanzarAcercaDe(null);
			break;
		case R.id.friends:
			lanzarFriends(null);
			break;
		case R.id.config:
			lanzarMisDatos(null);
			break;
		}
		return true;
	}

	public void lanzarAcercaDe(View view) {
		Intent i = new Intent(this, About.class);
		startActivity(i);
	}

	public void lanzarFriends(View view) {
		Intent i = new Intent(this, FriendsActivity.class);
		startActivity(i);
	}

	public void lanzarMisDatos(View view) {
		Intent i = new Intent(this, MyData.class);
		startActivity(i);
	}

	public void lanzarVistaLugar(View view) {
		final EditText entrada = new EditText(this);
		entrada.setText("0");
		new AlertDialog.Builder(this).setTitle("Selección del lugar")
				.setMessage("indicar su id:").setView(entrada)
				.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int whichButton) {
						long id = Long.parseLong(entrada.getText().toString());
						Intent i = new Intent(MainActivity.this,
								FriendView.class);
						i.putExtra("id", id);
						startActivity(i);
					}
				}).setNegativeButton("Cancelar", null).show();
	}

	protected void onListItemClick(ListView listView, View vista, int posicion,
			long id) {
		Intent i = new Intent(this, FriendView.class);
		i.putExtra("id", id);
		startActivityForResult(i, 1235);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == 1235) {
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		activarProveedores();
	}

	private void activarProveedores() {
		if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER,
					20 * 1000, 5, this);
		}
		if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
					10 * 1000, 10, this);
		}
	}

	@Override
	protected void onPause() {
		super.onPause();
		manejador.removeUpdates(this);
	}

	@Override
	public void onLocationChanged(Location location) {
		Log.d(Friends.TAG, "Nueva localización: " + location);
		actualizaMejorLocaliz(location);
	}

	@Override
	public void onProviderDisabled(String proveedor) {
		Log.d(Friends.TAG, "Se deshabilita: " + proveedor);
		activarProveedores();
	}

	@Override
	public void onProviderEnabled(String proveedor) {
		Log.d(Friends.TAG, "Se habilita: " + proveedor);
		activarProveedores();
	}

	@Override
	public void onStatusChanged(String proveedor, int estado, Bundle extras) {
		Log.d(Friends.TAG, "Cambia estado: " + proveedor);
		activarProveedores();
	}

	private static final long DOS_MINUTOS = 2 * 60 * 1000;

	private void actualizaMejorLocaliz(Location localiz) {

		if (localiz != null
				&& (mejorLocaliz == null
						|| localiz.getAccuracy() < 2 * mejorLocaliz
								.getAccuracy() || localiz.getTime()
						- mejorLocaliz.getTime() > DOS_MINUTOS)) {
			Log.d(Friends.TAG, "Nueva mejor localización");
			mejorLocaliz = localiz;

			Friends.posicionActual.setLongitud(localiz.getLongitude());
			Friends.posicionActual.setLatitud(localiz.getLatitude());

			misDatos.setPosicion(new GeoPunto(localiz.getLongitude(), localiz
					.getLatitude()));

			almacen.addPosition(misDatos.getNombre(), misDatos.getDireccion(),
					Double.toString(misDatos.getPosicion().getLongitud()),
					Double.toString(misDatos.getPosicion().getLatitud()));
			//
			Toast.makeText(
					this,
					"Enviando posicion "
							+ Double.toString(misDatos.getPosicion()
									.getLongitud())
							+ ":"
							+ Double.toString(misDatos.getPosicion()
									.getLatitud()), Toast.LENGTH_SHORT).show();
			Log.e("guadar desde main ",
					misDatos.getNombre()
							+ misDatos.getDireccion()
							+ Double.toString(misDatos.getPosicion()
									.getLongitud())
							+ Double.toString(misDatos.getPosicion()
									.getLatitud()));

			editor = pref.edit();
			SharedPreferences.Editor editor = pref.edit();
			editor.putFloat("longitud", (float) misDatos.getPosicion()
					.getLongitud());
			editor.putFloat("latitud", (float) misDatos.getPosicion()
					.getLatitud());
			editor.commit();
		}
	}

	@Override
	public void onInfoWindowClick(Marker marker) {
		// TODO Auto-generated method stub
		for (int id = 0; id < Friends.vectorFriends.size(); id++) {
			if (Friends.vectorFriends.get(id).getNombre()
					.equals(marker.getTitle())) {
				Intent intent = new Intent(this, FriendView.class);
				intent.putExtra("id", (long) id);
				startActivity(intent);
				break;
			}
		}
	}

	public void moveCamera(View view) {
		if (usuario) {
			mapa.moveCamera(CameraUpdateFactory.newLatLng(new LatLng(
					(long) misDatos.getPosicion().getLongitud(),
					(long) misDatos.getPosicion().getLatitud())));
		}
	}

	public void animateCamera(View view) {
		if (mapa.getMyLocation() != null)
			mapa.animateCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(
					mapa.getMyLocation().getLatitude(), mapa.getMyLocation()
							.getLongitude()), 30));
	}

	public void addMarker(View view) {
		mapa.addMarker(new MarkerOptions().position(mapa.getCameraPosition().target));
	}

	public void onMapClick(LatLng puntoPulsado) {
		mapa.addMarker(new MarkerOptions().position(puntoPulsado).icon(
				BitmapDescriptorFactory
						.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)));
	}
}
